@@ -18,11 +18,11 @@ sys.path.insert(0, os.path.abspath('../src'))
 # -- Project information -----------------------------------------------------
 
 project = 'PyFWI'
-copyright = '2022, Amir'
+copyright = '2022, Amir Mardan'
 author = 'Amir Mardan'
 
 # The full version, including alpha/beta/rc tags
-release = '0.0.4'
+release = '0.1.4'
 
 
 # -- General configuration ---------------------------------------------------
@@ -30,7 +30,7 @@ release = '0.0.4'
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.napoleon'
+extensions = ['sphinx.ext.autodoc', 'sphinx.ext.napoleon', 'nbsphinx'
 ]
 
 # Add any paths that contain templates here, relative to this directory.
