@@ -45,8 +45,8 @@ class FWI(Wave):
         if param_functions is None:
             self.dict2vec = tools.vel_dict2vec
             self.vec2dict = tools.vec2vel_dict
-            self.to_dv = lambda a: a
-            self.grad_from_dv = lambda a, b: a
+            self.to_dv = lambda a, param_functions_args: a
+            self.grad_from_dv = lambda a, param_functions_args, b: a
             self.param_functions_args = []
         else:
             self.dict2vec = param_functions['dict2vec']
@@ -64,6 +64,17 @@ class FWI(Wave):
         except:
             self.sd = 1.0
         
+        try:
+            self.param_relation = inpa['param_relation']
+        except:
+            self.param_relation = {}
+        
+        try:
+            self.prior_model = inpa['prior_model']
+        except:
+            self.prior_model = None
+            
+            
         # Dictionnary for TV regularization
         if 'tv' in keys:
             self.tv_properties = inpa['tv']
@@ -125,6 +136,13 @@ class FWI(Wave):
 
         c = 0
         rms = []
+        
+        k_0 = k_0 - 1
+        k_end -= 1
+        
+        if k_0 + n_params > k_end:
+            raise Exception("k_0 + n_params can't be larger than k_end!!")
+        
         for freq in freqs:
             m, rms0 = eval(method)(m, iter[c], freq, n_params, k_0, k_end)
             
@@ -145,7 +163,7 @@ class FWI(Wave):
 
         return method
 
-    def lbfgs(self, m0, ITER, freq, n_params=1, k0=0, k_end=1):
+    def lbfgs(self, m0, ITER, freq, n_params=1, k0=1, k_end=2):
         
         n_element = self.nz * self.nx
         mtotal = np.copy(m0)
@@ -154,8 +172,8 @@ class FWI(Wave):
 
         fun = MemoizeJac(self.fprime_single)
         jac = fun.derivative
-
-        for k in np.arange(k0-1, k_end-1, n_params):
+        
+        for k in np.arange(k0, k_end, n_params):
             print(f'Parameter number {k + 1: } to {k + n_params: }')
 
             m_1 = mtotal[:k * n_element]
@@ -165,7 +183,7 @@ class FWI(Wave):
             m_opt, hist, d = fmin_l_bfgs_b(fun, m_opt, jac, args=[m_1, m1, freq],
                                            m=10, factr=1e7, pgtol=1e-8, iprint=99,
                                            bounds=None, maxfun=15000, maxiter=ITER,
-                                           disp=None, callback=None, maxls=15)
+                                           disp=None, callback=None, maxls=10)
 
             # print(m_opt.max(), m_opt.min())
             rms_hist.append(hist)
@@ -185,9 +203,7 @@ class FWI(Wave):
         rms_data, adj_src = tools.cost_seismic(d_est, self.d_obs, fun=self.CF,
                                                fn=self.fn, freq=freq, order=3, axis=1
                                                )
-
-        rms = rms_data
-
+        
         grad_dv = self.gradient(adj_src, parameterization='dv')
         grad = self.grad_from_dv(grad_dv, self.param_functions_args, m_old)
         
@@ -197,26 +213,36 @@ class FWI(Wave):
         grad[params[2]] *= self.grad_coeff[2]
         
         grad = self.dict2vec(grad)
- 
+        rms = rms_data 
         return rms, grad
     
     def fprime_single(self, m0, m_1, m1, freq):
         mtotal = np.hstack((m_1, m0, m1))
         shape_1 = np.shape(m_1)[0]
         shape0 = np.shape(m0)[0]
-
-        rms_data, grad_model = self.fprime(mtotal, freq)
+        
+        k0 = np.int32(shape_1/self.n_elements)
+        kend = np.int32(k0 + shape0/self.n_elements)
+        
+        rms_data, grad_data = self.fprime(mtotal, freq)
         
         rms_reg, grad_reg = self.regularization.cost_regularization(m0,
                                                   tv_properties=self.tv_properties,
                                                   tikhonov_properties=self.tikhonov_properties
                                                   )
         
-        rms = rms_data + rms_reg
-        grad = grad_model[shape_1: shape_1 + shape0] + grad_reg
+        rms_model_relation, grad_model_relation = self.regularization.parameter_relation(mtotal, self.param_relation, k0, kend, freq)
+        
+        rms_mp, grad_mp = self.regularization.priori_regularization(m0, self.prior_model, k0, kend, freq)
+        
+        rms = rms_data + rms_reg + rms_model_relation + rms_mp
+        grad = grad_data[shape_1: shape_1 + shape0] + \
+            grad_reg + \
+            grad_model_relation[shape_1: shape_1 + shape0] + \
+            grad_mp
         
         print(m0.min(), m0.max())
-        print(" for f= {}: rms is: {} with rms_reg: {}, and rms_data: {}".format(freq, rms, rms_reg, rms_data))
+        print(" for f= {}: rms is: {} with rms_reg: {}, and rms_data: {}, rms_mp: {}, rms_model_relation: {}".format(freq, rms, rms_reg, rms_data, rms_mp, rms_model_relation))
 
         return rms, grad
     
\ No newline at end of file
