@@ -12,6 +12,7 @@ import scipy.sparse as sp
 from PyFWI.seismic_io import load_mat
 from PyFWI import rock_physics as rp 
 import PyFWI.processing as seis_process
+from scipy.ndimage import gaussian_filter
 
 def derivative(nx, nz, dx, dz, order):
     """
@@ -311,7 +312,117 @@ class regularization:
 
         return rms, grad
 
+    def parameter_relation(self, m0, models, k0, kend, freq):
+        """
+        parameter_relation considers regularization for the 
+        relation between parameters.
+        
 
+        Parameters
+        ----------
+        m0 : ndarray
+            Vector of parameters
+        models : dict
+            A dictionary containing couple of dictionaries which includes a numpy 
+            polyfit model and regularization parameter.
+        k0 : int
+            Index of the first parameter in m0
+        kend : int
+            Index of the last parameter in m0
+             
+
+        Returns
+        -------
+        rms : float
+            rms of regularization
+        grad: ndarray
+            Vector of gradient od the regularization
+        """
+        rms = 0
+        grad = np.zeros(m0.shape)
+        
+        for param in models:
+            par = [char for char in param]
+            model = models[param]['model']
+            lam = models[param]['lam']
+            
+            desired_freq  = models[param]['freqs']
+        
+            if freq not in np.array(desired_freq).reshape(-1): 
+                # has to be written like that to work eaither if freq is given as int or list
+                return 0.0, grad
+            
+            par_int = np.int32(par)
+            if par_int[1] in np.arange(k0+1, kend+1):
+                pre21 = model(m0[(par_int[0]-1) * self.n_elements:par_int[0] * self.n_elements])
+            
+                dm21 = m0[(par_int[1]-1)  * self.n_elements:par_int[1] * self.n_elements] - pre21  
+            
+                rms += 0.5 * lam * np.dot(dm21.T, dm21)
+                grad[(par_int[1]-1)  * self.n_elements: par_int[1] *self.n_elements] = gaussian_filter(lam * dm21 * 1, 1)
+                
+        return rms, grad
+    
+    def priori_regularization(self, m0, regularization_dict, k0, kend, freq):
+        """
+        priori_regularization consider the priori information regularization.
+        
+
+        Parameters
+        ----------
+        m0 : float
+            Vector of parameters
+        regularization_dict : dict
+            A dictionary containing couple of priori model and regularization hyperparameter
+        k0 : int
+            Index of the first parameter in m0
+        kend : int
+            Index of the last parameter in m0
+             
+        Returns
+        -------
+        rms : float
+            rms of regularization
+        grad: ndarray
+            Vector of gradient od the regularization
+            
+        References
+        -----------
+        Asnaashari et al., 2013, Regularized seismic full waveform inversion with prior model information, Geophysics, 78(2), R25-R36, eq. 5.
+        """
+        if regularization_dict is None:
+            return 0.0, np.zeros(m0.shape, np.float64)
+        
+        m0 = np.copy(m0[: kend * self.n_elements])
+        mp = np.zeros(m0.shape)
+        desired_freq  = regularization_dict['freqs']
+        
+        if freq not in np.array(desired_freq).reshape(-1):
+            return 0.0, np.zeros(m0.shape, np.float64)
+        
+        lam = regularization_dict['lam']
+        
+        mp_dict = regularization_dict['mp']
+        
+        for i in range(kend - k0):
+            mp[i * self.n_elements: (i + 1) * self.n_elements] = mp_dict[[*mp_dict][i]].reshape(-1)
+                    
+        ii = jj = np.arange((kend - k0) * self.n_elements)
+        v = np.ones((ii.shape)) / np.var(mp)
+        
+        W = sp.csr_matrix((v, (ii, jj)))
+        
+        diff = (m0 - mp).reshape(-1, 1) 
+                
+        rms = lam * 0.5 * (diff.T @ W) @ diff
+        
+        grad = lam * W.T @ diff
+        
+        return rms.item(), grad.reshape(-1)
+        
+         
+        
+    
 class fdm(object):
     def __init__(self, order):
         """
@@ -492,7 +603,7 @@ class fdm(object):
         error_x = np.sum(x * self.dxp(y, 1)) - np.sum(- self.dxm(x, 1) * y)
         error_z = np.sum(x * self.dzp(y, 1)) - np.sum(- self.dzm(x, 1) * y)
 
-        print(f"Errors for derivatives are \n {error_x = }, {error_z = }")    
+        print("Errors for derivatives are \n {}, {}".format(error_x, error_z))    
 
     def dt_computation(self, vp_max, dx, dz=None):
         '''
@@ -1404,11 +1515,11 @@ def cost_seismic(d_pre0, d_obs0, fun,
                                    params_oh=params_oh)
 
     adj_src = {}
-    adj_src['vx'] = adj_src_ndarray[0, :, :]
-    adj_src['vz'] = adj_src_ndarray[1, :, :]
-    adj_src['taux'] = adj_src_ndarray[2, :, :]
-    adj_src['tauz'] = adj_src_ndarray[3, :, :]
-    adj_src['tauxz'] = adj_src_ndarray[4, :, :]
+    adj_src['vx'] = adj_src_ndarray[0, :, :].astype(np.float32)
+    adj_src['vz'] = adj_src_ndarray[1, :, :].astype(np.float32)
+    adj_src['taux'] = adj_src_ndarray[2, :, :].astype(np.float32)
+    adj_src['tauz'] = adj_src_ndarray[3, :, :].astype(np.float32)
+    adj_src['tauxz'] = adj_src_ndarray[4, :, :].astype(np.float32)
 
     return rms, adj_src
 
