@@ -16,7 +16,7 @@ class Circular():
     def __init__(self, name):
         self.name = name
         
-    def louboutin(self, vintage, smoothing):
+    def louboutin(self, vintage=1, smoothing=0):
         """
         louboutin Generate perturbation model based on only vp.
 
@@ -77,7 +77,7 @@ class Circular():
                
         return model
 
-    def Hu_circles(self, vintage, smoothing):
+    def Hu_circles(self, vintage=1, smoothing=0):
         """
         Hu_circles a model including porosity, clay content, and saturation.
 
@@ -112,7 +112,7 @@ class Circular():
         return model
     
     
-    def perturbation_dv(self, vintage, smoothing):
+    def perturbation_dv(self, vintage=1, smoothing=0):
         """
         perturbation_dv creates perturbation model in different locations
 
@@ -148,7 +148,7 @@ class Laminar():
     def __init__(self,name) -> None:
         self.name = name
         
-    def Hu_laminar(self, vintage, smoothing):
+    def Hu_laminar(self, vintage=1, smoothing=0):
         # Based on Hu et al., 2020, Direct rock physics inversion 
         
         model = background((100, 100), {'phi':0.3, 'cc':0.1, 'sw':0.2})
@@ -163,7 +163,7 @@ class Laminar():
         return model
 
 
-    def dupuy(self, vintage, smoothing):
+    def dupuy(self, vintage=1, smoothing=0):
         # based on Dupuy et al, 2016, 
         # Estimation of rock physics properties from seismic attributes — Part 2: Applications
 
@@ -199,6 +199,14 @@ class Laminar():
 
 
 class ModelGenerator(Circular, Laminar):
+    """
+    ModelGenerator provides synthetic models for your research.
+
+    Parameters
+    ----------
+    name : str
+        Name of the desired model.
+    """
 
     def __init__(self, name):
         Circular.__init__(self, name)
@@ -234,7 +242,23 @@ class ModelGenerator(Circular, Laminar):
         ax.imshow(self.model[property[0]])
         
     
-    def marmousi(self, vintage, smoothing):
+    def marmousi(self, vintage=1, smoothing=0):
+        """
+        marmousi method generates the Maemousi-2 model.
+
+
+        Parameters
+        ----------
+        vintage : int
+            Baseline (1) or monitor (2) model.
+        smoothing : float
+            If you need to get the smooth version of the model, specify the smoothness 
+
+        Returns
+        -------
+        model : dict
+            A dictionary containing Vp, Vs, and density.
+        """
         path = os.path.dirname(__file__) + '/data/'
         
         if os.path.exists(path) is False:
@@ -447,6 +471,26 @@ def add_circle (model, circle_prop, r, cx, cz):
 
 
 def model_smoother(model, smoothing_value, constant_surface=0, constant_left=0):
+    """
+    model_smoother smoothens all parameters of inside model.
+
+
+    Parameters
+    ----------
+    model : dict
+        Dictionary of true model.
+    smoothing_value : float
+        Smoothness value
+    constant_surface : int, optional
+        Number of rows untouched for surface acquisition, by default 0
+    constant_left : int, optional
+        Number of columns untouched for cross-well acquisition, by default 0
+
+    Returns
+    -------
+    dict
+        Smoothed model
+    """
     model0 = {}
     for params in model:
         model0[params] = gaussian_filter(model[params], smoothing_value)
@@ -454,6 +498,7 @@ def model_smoother(model, smoothing_value, constant_surface=0, constant_left=0):
         model0[params][:, :constant_left] = model[params][:, :constant_left]
     return model0
 
+
 def pcs_perturbation(rho=None, prop_back=None, prop_circle=None, nz=100, nx=100, r=8, monitor=False):
     """
     pcs_perturbation a model including porosity, clay content, and saturation.
@@ -530,6 +575,31 @@ def pcs_perturbation(rho=None, prop_back=None, prop_circle=None, nz=100, nx=100,
 
 
 def model_resizing(model0,  bx=None, ex=None, bz=None, ez=None, ssr=(1, 1)):
+    """
+    model_resizing resizes your model which is dictionary by cutting and interpolation.
+
+    
+
+    Parameters
+    ----------
+    model0 : dict
+        The input model to be resized
+    bx : int, optional
+        First column of desired model, by default None
+    ex : int, optional
+        Last column of desired model, by default None
+    bz : int, optional
+        First row of desired model, by default None
+    ez : int, optional
+        Last row of desired model, by default None
+    ssr : tuple, optional
+        Sampling rate for interpolation in Z- and X-directions, by default (1, 1)
+
+    Returns
+    -------
+    model : dict
+        Dictionary containg the resized model
+    """
     model = copy.deepcopy(model0)
     for param in model:
         gz, gx = np.mgrid[:model[param].shape[0], :model[param].shape[1]]
