@@ -83,14 +83,34 @@ class wave_preparation():
         
         self.data_guide_sampling = acq.discretized_acquisition_plan(data_guide, self.dh, inpa['npml'])
 
+        self.rec_top_left_const = np.int32(0)
+        self.rec_top_left_var = np.int32(0)
+        self.rec_top_right_const = np.int32(0)
+        self.rec_top_right_var = np.int32(0)
+        self.rec_surface_const = np.int32(0)
+        self.rec_surface_var = np.int32(0)
+               
         if inpa["acq_type"] == 0:
-            self.rec_cts = np.int32(rec_loc[0, 0] / self.dh + inpa['npml'])
-            self.rec_var = np.int32(rec_loc[:, 1] / self.dh + inpa['npml'])[0]
+            self.rec_top_right_const = np.int32(rec_loc[0, 0] / self.dh + inpa['npml'])
+            self.rec_top_right_var = np.int32(rec_loc[:, 1] / self.dh + inpa['npml'])[0]
             self.src_cts = src.i[0]
             
-        elif inpa["acq_type"] in [1, 2]:
-            self.rec_cts = np.int32(rec_loc[0, 1] / self.dh + inpa['npml'])
-            self.rec_var = np.int32(rec_loc[:, 0] / self.dh + inpa['npml'])[0]
+        elif inpa["acq_type"] == 1:
+            self.rec_surface_const = np.int32(rec_loc[0, 1] / self.dh + inpa['npml'])
+            self.rec_surface_var = np.int32(rec_loc[:, 0] / self.dh + inpa['npml'])[0]
+            self.src_cts = src.j[0]
+            
+        elif inpa["acq_type"] == 2:
+            a = np.int32(rec_loc[-self.n_well_rec, :]/ self.dh + inpa['npml'])
+            self.rec_top_right_const = np.copy(a[0])
+            self.rec_top_right_var = np.copy(a[1])
+            
+            a = np.int32(rec_loc[self.n_well_rec, :]/ self.dh + inpa['npml'])
+            self.rec_surface_const = np.copy(a[1])
+            self.rec_surface_var = np.copy(a[0])
+            
+            self.rec_top_left_const = np.int32(rec_loc[0, 0] / self.dh + inpa['npml'])
+            self.rec_top_left_var = np.int32(rec_loc[:, 1] / self.dh + inpa['npml'])[0]
             self.src_cts = src.j[0]
 
         # ======== Parameters Boundary condition ======
@@ -434,12 +454,22 @@ class wave_preparation():
             #define left4   (i)*Nx + (j-4)
             
             #define src_depth %d
+            #define rec_top_left_const  %d
+            #define rec_top_left_var %d
+            #define rec_top_right_const %d
+            #define rec_top_right_var %d
+            #define rec_surface_const %d
+            #define rec_surface_var %d
+            
 
                 //float Diff_Forward(float , float , float , float , int );
                 //float Diff_Backward(float , float , float , float , int );
         """ % (self.tnz, self.tnx, self.ns, self.dt, self.dh, self.dh, self.npml,
                self.n_surface_rec, self.n_well_rec, self.sdo,
-               c1, c2, c3, c4, self.src_cts)
+               c1, c2, c3, c4, self.src_cts,
+               self.rec_top_left_const, self.rec_top_left_var,
+               self.rec_top_right_const, self.rec_top_right_var,
+               self.rec_surface_const, self.rec_surface_var)
 
         # Decide on openCl file based on medium type
         cl_file_name = "elastic.cl"
@@ -544,10 +574,32 @@ class wave_preparation():
         
         
 class wave_propagator(wave_preparation):
-    def __init__(self, inpa, src, rec_loc, model_shape, n_well_rec=None, chpr=10, components=4):
+    """
+    wave_propagator is a class to handle the forward modeling and gradient calculation.
+
+    [extended_summary]
+
+    Parameters
+    ----------
+    inpa : dict
+        A dictionary including most of the required inputs
+    src : class
+        Source object
+    rec_loc : ndarray
+        Location of the receivers
+    model_shape : tuple
+        Shape of the model
+    n_well_rec: int
+        Number of receivers in the well
+    chpr : percentage
+        Checkpoint ratio in percentage
+    component:
+        Seismic output
+    """
+    def __init__(self, inpa, src, rec_loc, model_shape, n_well_rec=None, chpr=10, components=0):
         wave_preparation.__init__(self, inpa, src, rec_loc, model_shape, n_well_rec, chpr=chpr, components=components)
     
-    def forward_propagator(self, model, W=None, grad=None):
+    def forward_propagator(self, model):
         """ This function is in charge of forward modelling for acoustic case
 
         Parameters
@@ -568,7 +620,10 @@ class wave_propagator(wave_preparation):
                                  )
             
             self.__kernel(s, coeff) 
-            
+        
+        if self.acq_type == 2:
+            for par in self.seismogram:
+                self.seismogram[par][:, :self.n_well_rec] = np.flip(self.seismogram[par][:, :self.n_well_rec], axis=1)
         return self.seismogram
             
     def __kernel(self, s, coeff=+1):
@@ -582,7 +637,7 @@ class wave_propagator(wave_preparation):
                             self.taux_b, self.tauz_b, self.tauxz_b,
                             self.seismogramid_vx_b, self.seismogramid_vz_b,
                             self.seismogramid_taux_b, self.seismogramid_tauz_b, self.seismogramid_tauxz_b,
-                            self.dxr, self.rec_cts, self.rec_var,
+                            self.dxr,
                             self.srcx[s], self.srcz[s],
                             src_kt_x, src_kt_z)
             
@@ -729,8 +784,7 @@ class wave_propagator(wave_preparation):
                                 self.ataux_b, self.atauz_b, self.atauxz_b,
                                 self.res_vx_b, self.res_vz_b,
                                 self.res_taux_b, self.res_tauz_b, self.res_tauxz_b,
-                                self.dxr, self.rec_cts,
-                                self.rec_var, self.rec_var)
+                                self.dxr)
                                 
             self.prg.Adj_update_tau(self.queue, (self.tnz, self.tnx), None,
                                     self.avx_b, self.avz_b,
@@ -758,14 +812,30 @@ class wave_propagator(wave_preparation):
             
             # Plotting wave propagation
             if self.backward_show and (np.remainder(t, 20) == 0 or t == self.nt - 2):
-                cl.enqueue_copy(self.queue, vx_show, self.vx_b)
+                cl.enqueue_copy(self.queue, vx_show, self.taux_b)
 
-                cl.enqueue_copy(self.queue, adj_vx_show, self.avx_b)
+                cl.enqueue_copy(self.queue, adj_vx_show, self.ataux_b)
 
                 self.plot_propagation(vx_show, t, adj_vx_show)
                 
             
-    def forward_modeling(self, model0, show=False, W=None, grad=None):
+    def forward_modeling(self, model0, show=False):
+        """
+        forward_modeling performs the forward modeling.
+
+
+        Parameters
+        ----------
+        model0 : dict
+            The earth model
+        show : bool, optional
+            True if you desire to see the propagation of the wave, by default False
+
+        Returns
+        -------
+        dict
+            Seismic section
+        """
         self.forward_show = show
         model = model0.copy()
 
@@ -775,14 +845,36 @@ class wave_propagator(wave_preparation):
         
         self.pml_preparation(model['vp'].max())
         self.elastic_buffers(model)
-        seismo = self.forward_propagator(model, W, grad) 
+        seismo = self.forward_propagator(model) 
         data = acq.seismic_section(seismo, self.components)
         return data
     
-    def gradient(self, res, show=False, Lam=None, grad=None, parameterization='dv'):
+    def gradient(self, res, show=False, parameterization='dv'):
+        """
+        gradient estimates the gradient using adjoint-state method.
+
+        Parameters
+        ----------
+        res : dict
+            The adjoint of the derivative of ost function with respect to wavefield
+        show : bool, optional
+            True if you desire to see the backward wave propagation, by default False
+        parameterization : str, optional
+            Specify the parameterization for output, by default 'dv'
+
+        Returns
+        -------
+        dict
+            Gradient
+        """
         self.backward_show = show
         self.adjoint_buffer_preparing()
         
+        # To reorder the receivers in the left well
+        if self.acq_type == 2:
+            for par in self.seismogram:
+                self.seismogram[par][:, :self.n_well_rec] = np.flip(self.seismogram[par][:, :self.n_well_rec], axis=1)
+        
         res = acq.prepare_residual(res, 1)
         if show:
             self.initial_wavefield_plot({'vp':self.vp}, plot_type="Backward")
@@ -846,7 +938,7 @@ if __name__ == "__main__":
     dt = D.dt_computation(vp.max(), inpa['dh'])
     inpa['dt'] = dt
     
-    print(f'{dh = } ........... {dt = }')
+    # print(f'{dh = } ........... {dt = }')
     inpa['t'] = 0.36
     
     offsetx = inpa['dh'] * model_shape[1]
@@ -878,15 +970,6 @@ if __name__ == "__main__":
         rms, res = CF(d_est, d_obs)
 
         print(rms)
-    
-        # fig = plt.figure()
-        # ax = fig.add_subplot(1, 3, 1)
-        # seiplt.seismic_section(ax, d_obs['taux'], vmin=d_obs['taux'].min() / 5, vmax=d_obs['taux'].max() / 5)
-        # ax = fig.add_subplot(1, 3, 2)
-        # seiplt.seismic_section(ax, d_est['taux'])
-        # ax = fig.add_subplot(1, 3, 3)
-        # seiplt.seismic_section(ax, res['taux'])
-        # plt.show()
         
         grad = Lam.gradient(res, show=False)
         
